/*
 * Project name:
     RTC3 click (Demonstration of mikroBUS RTC3 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150401:
     - initial release (DM);
 * Description:
      This project is simple demonstration how to read/write date and time from
      BQ32000DR RTC (real-time clock).
      Date and time are read from the RTC every 10 milisecond and printed on LCD.
      Pressing A0 button will write default time and date to RTC module.
      When button is released RTC will start counting.
      User can change default time and date in main function.
 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev.Board:       EasyAVR7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    RTC3 click board click
                      http://www.mikroe.com/click/RTC3/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/

 * NOTES:
     - Turn on Lcd backlight switch SW3.1 (board specific)
     - Place Pull down PA0 (tree state switch), and SW1 (Button press level)
       to PA position
     - Place RTC3 click board with DS1307 to the mikroBUS socket 1.
     - Place power selection jumper (J5) in according to setting on RTC3 Click
     - Turn on SW6.1 and SW6.2
 */
 
// Lcd module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End Lcd module connections

// Software I2C connections
sbit Soft_I2C_Scl_Output    at PORTB2_bit;
sbit Soft_I2C_Sda_Output    at PORTB0_bit;
sbit Soft_I2C_Scl_Input     at PINB2_bit;
sbit Soft_I2C_Sda_Input     at PINB0_bit;
sbit Soft_I2C_Scl_Direction at DDB2_bit;
sbit Soft_I2C_Sda_Direction at DDB0_bit;
// End Software I2C connections

// Lcd constants
char txt1[] = "mikroElektronika";
char txt2[] = "RTC3 click";
char txt3[] = "DATE:";
char txt4[] = "TIME:";

// Module variables
unsigned char hours, minutes, seconds, day, week, month, year;    // Global date/time variables
char oldstate = 0;

// RTC Definitions
#define RTC_ADDR  0xD0

/*******************************************************************************
* BQ32000DR  Functions
*******************************************************************************/

/*******************************************************************************
* Read data from RTC BQ32000DR
* input : addres of RTC register
* output: value of of RTC register
*******************************************************************************/
unsigned char RTC_Read(unsigned char addr){
  unsigned char value;

  Soft_I2C_Start();                   // Issue start signal
  Soft_I2C_Write(RTC_ADDR);              // Address BQ32000DR , see BQ32000DR  datasheet
  Soft_I2C_Write(addr);                  // Start from address 2
  Soft_I2C_Start();                   // Issue repeated start signal
  Soft_I2C_Write(RTC_ADDR);              // Address BQ32000DR  for reading R/W=1

  value = Soft_I2C_Read(0);             // Read seconds byte
  Soft_I2C_Stop();                    // Issue stop signal

  return value;
}

/*******************************************************************************
* Write data from BQ32000DR
* input : addres of RTC register, value of of RTC register
*******************************************************************************/
void RTC_Write(unsigned char addr, unsigned char value){
  Soft_I2C_Start();            // Issue start signal
  Soft_I2C_Write(RTC_ADDR);    // Address BQ32000DR
  Soft_I2C_Write(addr);        // Start from address
  Soft_I2C_Write(value);       // Write value to RTC register
  Soft_I2C_Stop();             // Issue stop signal
}

/*******************************************************************************
* Read time from RTC BQ32000DR
* input : pointer to variables where RTC data will be stored
* output: variables with RTC data
*******************************************************************************/
void Read_Time(unsigned char *p_hours, unsigned char *p_minutes, unsigned char *p_seconds,
               unsigned char *p_day, unsigned char *p_week, unsigned char *p_month, unsigned char *p_year){
  Soft_I2C_Start();                   // Issue start signal
  Soft_I2C_Write(RTC_ADDR);           // Address BQ32000DR , see BQ32000DR  datasheet
  Soft_I2C_Write(0);                  // Start from address 0
  Soft_I2C_Start();                   // Issue repeated start signal
  Soft_I2C_Write(RTC_ADDR + 1);       // Address BQ32000DR  for reading R/W=1

  *p_seconds = Soft_I2C_Read(1);      // Read seconds byte
  *p_minutes = Soft_I2C_Read(1);      // Read minutes byte
  *p_hours = Soft_I2C_Read(1);        // Read hours byte
  *p_week =Soft_I2C_Read(1);
  *p_day =Soft_I2C_Read(1);
  *p_month =Soft_I2C_Read(1);
  *p_year =Soft_I2C_Read(0);

  Soft_I2C_Stop();                  // Issue stop signal
}

/*******************************************************************************
* Write time to RTC BQ32000DR
* input : variables with RTC data
*******************************************************************************/
void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds,
                unsigned char c_day, unsigned char c_week, unsigned char c_month, unsigned char c_year){
   Soft_I2C_Start();                     // issue start signal
   Soft_I2C_Write(RTC_ADDR);             // address BQ32000DR
   Soft_I2C_Write(0);                    // start from word at address (REG0)
   Soft_I2C_Write(0x80);                 // write $80 to REG0. (pause counter + 0 sec)

   Soft_I2C_Write(c_minutes);            // write 0 to minutes word to (REG1)
   Soft_I2C_Write(c_hours);              // write 17 to hours word (24-hours mode)(REG2)
   Soft_I2C_Write(c_week);               // write 2 - Monday (REG3)
   Soft_I2C_Write(c_day);                // write 4 to date word (REG4)
   Soft_I2C_Write(c_month);              // write 5 (May) to month word (REG5)
   Soft_I2C_Write(c_year);               // write 01 to year word (REG6)
   Soft_I2C_Stop();                      // issue stop signal

   Soft_I2C_Start();                     // issue start signal
   Soft_I2C_Write(RTC_ADDR);             // address BQ32000DR
   Soft_I2C_Write(0);                    // start from word at address 0
   Soft_I2C_Write(0 | c_seconds);        // write 0 to REG0 (enable counting + 0 sec)
   Soft_I2C_Stop();                      // issue stop signal
}

/*******************************************************************************
* Show on the LCD display
* input : variables with RTC data
*******************************************************************************/
void Show_Time(){
  char *txt;

  seconds  =  ((seconds & 0x70) >> 4)*10 + (seconds & 0x0F);
  minutes  =  ((minutes & 0x70) >> 4)*10 + (minutes & 0x0F);
  hours    =  ((hours & 0x30) >> 4)*10 + (hours & 0x0F);
  week     =  (week & 0x07);
  day      =  ((day & 0xF0) >> 4)*10 + (day & 0x0F);
  month    =  ((month & 0x10) >> 4)*10 + (month & 0x0F);
  year     =  ((year & 0xF0)>>4)*10+(year & 0x0F);

  switch(week)
  {
    case 1: txt="Sun"; break;
    case 2: txt="Mon"; break;
    case 3: txt="Tue"; break;
    case 4: txt="Wed"; break;
    case 5: txt="Thu"; break;
    case 6: txt="Fri"; break;
    case 7: txt="Sat"; break;
  }

  LCD_Out(1,1, txt);
  Lcd_Chr(1, 6, (day / 10)   + 48);    // Print tens digit of day variable
  Lcd_Chr(1, 7, (day % 10)   + 48);    // Print oness digit of day variable
  Lcd_Chr(1, 9, (month / 10) + 48);
  Lcd_Chr(1,10, (month % 10) + 48);
  Lcd_Chr(1,14, (year / 10)  + 48);
  Lcd_Chr(1,15, (year % 10)  + 48);

  Lcd_Chr(2, 6, (hours / 10)   + 48);
  Lcd_Chr(2, 7, (hours % 10)   + 48);
  Lcd_Chr(2, 9, (minutes / 10) + 48);
  Lcd_Chr(2,10, (minutes % 10) + 48);
  Lcd_Chr(2,12, (seconds / 10) + 48);
  Lcd_Chr(2,13, (seconds % 10) + 48);
}

/*******************************************************************************
* MAIN PROGRAM
*******************************************************************************/
void main(){
  // Configure user buttons
  DDA0_bit = 0;                      // Set PA0 pin as input.
                                     // PA0 is used to write inital data to RTC.
  Soft_I2C_Init();                   // Initialize I2C bus for communication with RTC
  Lcd_Init();                        // Initialize Lcd
  
  // charging super - capacitor
  Soft_I2C_Start();
  Soft_I2C_Write(RTC_ADDR);
  Soft_I2C_Write(0x08);
  Soft_I2C_Write(0x20);
  Soft_I2C_Write(0x45);
  Soft_I2C_Stop;

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
  Lcd_Out(1,1,txt1);                 // Write text in first row
  Lcd_Out(2,4,txt2);                 // Write text in second row

  Delay_ms(2000);                    // delay 2 sec

  Lcd_Cmd(_LCD_CLEAR);               // Prepare and output static text on LCD
  LCD_Chr(1,8,'.');
  LCD_Chr(1,11,'.');
  LCD_Out(2,1,"Time:");
  LCD_Chr(2,8,':');
  LCD_Chr(2,11,':');
  LCD_Out(1,12,"20");

  // Start the test examples
  while(1){                           // Endless loop
    if (Button(&PINA, 0, 1, 0)) {               // Detect logical one
      oldstate = 1;                              // Update flag
    }
    if (oldstate && Button(&PINA, 0, 1, 1)) {   // Detect one-to-zero transition
    // Write TIME    16  : 30  : 00  : 1.  Wednesday april  2015
      Write_Time(0x16, 0x30, 0x00, 0x01, 0x04,   0x04, 0x15);
      oldstate = 0;                              // Update flag
    }
    Read_Time(&hours, &minutes, &seconds, &day, &week, &month, &year);
    Show_Time();
    Delay_mS(10);

  }
}